# TODO package level docs

from dfmpy.commands.add import add
from dfmpy.commands.initialize import initialize
from dfmpy.commands.ls import ls
from dfmpy.commands.uninstall import uninstall
from dfmpy.commands.sync import sync

# TODO docs
__all__ = tuple([
    'add'
    'initialize',
    'ls',
    'uninstall',
    'sync',
])
